#include <stdio.h>
#include <windows.h>
#include <Sql.h>
#include <Sqlext.h>
#include <Sqltypes.h>

/*-----------------------------------------------------------------------------
 * err_exit: Print error message and halt this program.
 *-----------------------------------------------------------------------------
 */

void err_exit(const char *msg, SQLSMALLINT handle_type, SQLHANDLE handle)
{
	SQLSMALLINT err_rec_no; /* Error record number */
	SQLCHAR msgbuffer[SQL_MAX_MESSAGE_LENGTH+1]; /* Error message buffer */
	SQLSMALLINT msglength; /* Actuall length of error message */
	SQLINTEGER sqlcode; /* Error code */
	SQLCHAR sqlstate[SQL_SQLSTATE_SIZE+1];

	/* Print Error Message: */
	printf("%s!\n", msg);

	/* Print System Error Diagnositics: */
	if(handle != SQL_NULL_HANDLE) {
		err_rec_no = 1;
		while(SQLGetDiagRec(handle_type, handle, err_rec_no, sqlstate,
			&sqlcode, msgbuffer, SQL_MAX_MESSAGE_LENGTH+1,
			&msglength) == SQL_SUCCESS) {
			sqlstate[SQL_SQLSTATE_SIZE] = '\0';
			printf("SQLCODE = %d, SQLSTATE = %s\n",
				sqlcode, sqlstate);
			printf("MESSAGE: %s\n", msgbuffer);
			err_rec_no++;
		}
	}

	/* Abort program: */
	exit(1);
}

/*-----------------------------------------------------------------------------
 * main: Small ODBC example: Execute SQL query and print results.
 *-----------------------------------------------------------------------------
 */

int main()
{
	SQLRETURN rc; /* Return Code */
	SQLHENV	env; /* Environment Handle */
	SQLHDBC conn; /* Database Connection Handle */
	SQLHSTMT stmt; /* Statement Handle */
	SQLINTEGER prod_id;
	SQLINTEGER id_null;
	SQLCHAR prod_name[50];
	SQLINTEGER name_null;

	/* Allocate Environment Handle: */
	rc = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &env);
	if(rc != SQL_SUCCESS)
		err_exit("Error allocating Environment Handle",
			(SQLSMALLINT)0, SQL_NULL_HANDLE);
	rc = SQLSetEnvAttr(env, SQL_ATTR_ODBC_VERSION, 
                 (SQLPOINTER) SQL_OV_ODBC3, (SQLINTEGER) 0);
	if(rc != SQL_SUCCESS)
		err_exit("Error setting ODBC version", SQL_HANDLE_ENV, env);

	/* Allocate Connection Handle: */
	rc = SQLAllocHandle(SQL_HANDLE_DBC, env, &conn);
	if(rc != SQL_SUCCESS)
		err_exit("Error allocating Connection Handle",
			SQL_HANDLE_ENV, env);

	/* Connect to Database: */
	rc = SQLConnect(conn,
		(SQLCHAR*) "Northwind", SQL_NTS, /* Data Source Name (Server)*/
		(SQLCHAR*) "", SQL_NTS, /* User */
		(SQLCHAR*) "", SQL_NTS); /* Password */
	if(rc != SQL_SUCCESS && rc != SQL_SUCCESS_WITH_INFO)
		err_exit("Error connecting to Database", SQL_HANDLE_DBC, conn);

	/* Allocate Statement Handle: */
	rc = SQLAllocHandle(SQL_HANDLE_STMT, conn, &stmt);
	if(rc != SQL_SUCCESS)
		err_exit("Error allocating Statement Handle",
			SQL_HANDLE_DBC, conn);

	/* Execute a Query: */
	rc = SQLExecDirect(stmt, (SQLCHAR*)
		"SELECT ProductID, ProductName FROM Products ORDER BY 1",
		SQL_NTS);
	if(rc != SQL_SUCCESS)
		err_exit("Error processing Query", SQL_HANDLE_STMT, stmt);

	/* Bind Variables to the Output Columns: */
	rc = SQLBindCol(stmt, 1, SQL_C_SLONG, (SQLPOINTER) &prod_id, 0,
		&id_null);
	if(rc != SQL_SUCCESS)
		err_exit("Error binding variable to ProductID column",
			SQL_HANDLE_STMT, stmt);
	rc = SQLBindCol(stmt, 2, SQL_C_CHAR, (SQLPOINTER) &prod_name,
		(SQLINTEGER) sizeof(prod_name), &name_null);
	if(rc != SQL_SUCCESS)
		err_exit("Error binding variable to ProductName column",
			SQL_HANDLE_STMT, stmt);

	/* Print Query Result: */
	rc = SQLFetch(stmt);
	if(rc == SQL_NO_DATA_FOUND)
		printf("0 Result Rows\n");
	else if(rc == SQL_SUCCESS) {
		while(rc == SQL_SUCCESS || rc == SQL_SUCCESS_WITH_INFO) {
			if(id_null == SQL_NULL_DATA)
				printf("(null) ");
			else
				printf("%6d ", prod_id);
			if(name_null == SQL_NULL_DATA)
				printf("(null)\n");
			else
				printf("%s\n", prod_name);
			rc = SQLFetch(stmt);
		}
	}
	else
		err_exit("Error fetching row", SQL_HANDLE_STMT, stmt);

	/* Free Handles, Close Database Connection: */
	rc = SQLFreeHandle(SQL_HANDLE_STMT, stmt);
	if(rc != SQL_SUCCESS)
		err_exit("Error freeing statement handle",
			SQL_HANDLE_STMT, stmt);
	rc = SQLDisconnect(conn);
	if(rc != SQL_SUCCESS)
		err_exit("Error disconnecting from the database",
			SQL_HANDLE_DBC, conn);
	rc = SQLFreeHandle(SQL_HANDLE_DBC, conn);
	if(rc != SQL_SUCCESS)
		err_exit("Error freeing connection handle",
			SQL_HANDLE_DBC, conn);
	rc = SQLFreeHandle(SQL_HANDLE_ENV, env);
	if(rc != SQL_SUCCESS)
		err_exit("Error freeing environment handle!",
			SQL_HANDLE_ENV, env);

	/* We are done: */
	printf("Query successfully completed!\n");
	return(0);
}

